@@ -79,6 +79,13 @@ class Project(models.Model):
         related_name='user_projects',
         through='ProjectRelationship',
     )
+    activity_group = models.ForeignKey(
+        'ActivityGroup',
+        related_name='activity_group',
+        null=True,
+        blank=True,
+        verbose_name="restrict activities to",
+    )
     type = models.ForeignKey(
         Attribute,
         limit_choices_to={'type': 'project-type'},
@@ -178,6 +185,33 @@ class Activity(models.Model):
         verbose_name_plural = 'activities'
 
 
+class HourGroup(models.Model):
+    """Activities that are bundled together for billing"""
+
+    name = models.CharField(max_length=255, unique=True)
+    activities = models.ManyToManyField(
+        Activity,
+        related_name='activity_bundle',
+    )
+    order = models.PositiveIntegerField(unique=True, blank=True, null=True)
+
+    def __unicode__(self):
+        return self.name
+
+
+class ActivityGroup(models.Model):
+    """Activities that are allowed for a project"""
+
+    name = models.CharField(max_length=255, unique=True)
+    activities = models.ManyToManyField(
+        Activity,
+        related_name='activity_group',
+    )
+
+    def __unicode__(self):
+        return self.name
+
+
 class Location(models.Model):
     name = models.CharField(max_length=255, unique=True)
     slug = models.CharField(max_length=255, unique=True)
@@ -186,6 +220,15 @@ class Location(models.Model):
         return self.name
 
 
+ENTRY_STATUS = (
+    ('unverified', 'Unverified',),
+    ('verified', 'Verified',),
+    ('approved', 'Approved',),
+    ('invoiced', 'Invoiced',),
+    ('not-invoiced', 'Not Invoiced',),
+)
+
+
 class EntryManager(models.Manager):
     def get_query_set(self):
         qs = super(EntryManager, self).get_query_set()
@@ -219,14 +262,6 @@ class EntryWorkedManager(models.Manager):
         return qs.exclude(project__in=projects.values())
 
 
-ENTRY_STATUS = (
-    ('unverified', 'Unverified',),
-    ('verified', 'Verified',),
-    ('approved', 'Approved',),
-    ('invoiced', 'Invoiced',),
-)
-
-
 class Entry(models.Model):
     """
     This class is where all of the time logs are taken care of
@@ -242,11 +277,18 @@ class Entry(models.Model):
         Location,
         related_name='entries',
     )
+    entry_group = models.ForeignKey(
+       'EntryGroup',
+        related_name='entries',
+        blank=True, null=True,
+        on_delete=models.SET_NULL,
+    )
     status = models.CharField(
         max_length=24,
         choices=ENTRY_STATUS,
         default='unverified',
     )
+
     start_time = models.DateTimeField()
     end_time = models.DateTimeField(blank=True, null=True)
     seconds_paused = models.PositiveIntegerField(default=0)
@@ -362,7 +404,31 @@ class Entry(models.Model):
                     '%(project)s - %(activity)s - ' % entry_data + \
                     'from %(start_time)s to %(end_time)s' % entry_data
                     raise ValidationError(output)
-
+        try:
+            activity_group = self.project.activity_group
+            if activity_group:
+                activity = self.activity
+                if not activity_group.activities.filter(pk=activity.pk).exists():
+                    name = activity.name
+                    err_msg = '%s is not allowed for this project. ' % name
+                    allowed = activity_group.activities.filter()
+                    allowed = allowed.values_list('name', flat=True)
+                    allowed_names = ['among ']
+                    if len(allowed) > 1:
+                        for index, activity in enumerate(allowed):
+                            allowed_names += activity
+                            if index < len(allowed) - 2:
+                                allowed_names += ', '
+                            elif index < len(allowed) - 1:
+                                allowed_names += ', and '
+                        allowed_activities = ''.join(allowed_names)
+                    else:
+                        allowed_activities = allowed[0]
+                    err_msg += 'Please choose %s' % allowed_activities
+                    raise ValidationError(err_msg)
+        except (Project.DoesNotExist, Activity.DoesNotExist):
+            # Will be caught by field requirements
+            pass
         if end <= start:
             raise ValidationError('Ending time must exceed the starting time')
         return True
@@ -498,6 +564,37 @@ class Entry(models.Model):
             ('view_entry_summary', 'Can view entry summary page'),
         )
 
+
+class EntryGroup(models.Model):
+    VALID_STATUS = ('invoiced', 'not-invoiced')
+    STATUS_CHOICES = [status for status in ENTRY_STATUS \
+                      if status[0] in VALID_STATUS]
+    user = models.ForeignKey(User, related_name='entry_group')
+    project = models.ForeignKey(Project, related_name='entry_group')
+    status = models.CharField(max_length=24, choices=STATUS_CHOICES,
+                              default='invoiced')
+    number = models.IntegerField("Reference #", blank=True, null=True)
+    comments = models.TextField(blank=True, null=True)
+    created = models.DateTimeField(auto_now_add=True)
+    modified = models.DateTimeField(auto_now=True)
+    start = models.DateField(blank=True, null=True)
+    end = models.DateField()
+
+    def delete(self):
+        self.entries.update(status='approved')
+        super(EntryGroup, self).delete()
+
+    def __unicode__(self):
+        invoice_data = {
+            'number': self.number,
+            'status': self.status,
+            'project': self.project,
+            'end': self.end.strftime('%b %Y'),
+        }
+        return 'Entry Group ' + \
+               '%(number)s: %(status)s - %(project)s - %(end)s' % invoice_data
+
+
 # Add a utility method to the User class that will tell whether or not a
 # particular user has any unclosed entries
 User.clocked_in = property(lambda user: user.timepiece_entries.filter(
@@ -768,6 +865,60 @@ class ProjectContract(models.Model):
         return unicode(self.project)
 
 
+class ContractMilestone(models.Model):
+    contract = models.ForeignKey(ProjectContract, related_name='milestones')
+    name = models.CharField(max_length=255)
+    start_date = models.DateField()
+    end_date = models.DateField()
+    hours = models.DecimalField(max_digits=8, decimal_places=2,
+                                default=0)
+
+    class Meta(object):
+        ordering = ('end_date',)
+
+    def hours_worked(self):
+        """Hours worked during this milestone"""
+        if not hasattr(self, '_hours_worked'):
+            self._hours_worked = Entry.objects.filter(
+                project=self.contract.project,
+                start_time__gte=self.start_date,
+                end_time__lt=self.end_date + datetime.timedelta(days=1),
+            ).aggregate(sum=Sum('hours'))['sum']
+        return self._hours_worked or 0
+
+    def total_budget(self):
+        """Total budget through the end of this milestone"""
+        if not hasattr(self, '_total_budget'):
+            end_date = self.end_date + datetime.timedelta(days=1)
+            previous = self.contract.milestones.filter(end_date__lt=end_date)
+            self._total_budget = previous.aggregate(sum=Sum('hours'))['sum']
+        return self._total_budget or 0
+
+    def total_hours_worked(self):
+        """Total hours worked on project through the end of this milestone"""
+        if not hasattr(self, '_total_hours_worked'):
+            self._total_hours_worked = Entry.objects.filter(
+                project=self.contract.project,
+                start_time__gte=self.contract.start_date,
+                end_time__lt=self.end_date + datetime.timedelta(days=1),
+            ).aggregate(sum=Sum('hours'))['sum']
+        return self._total_hours_worked or 0
+
+    def hours_remaining(self):
+        """Hours over the milestone budget"""
+        return self.hours - self.hours_worked()
+
+    def total_hours_remaining(self):
+        """Hours over the entire project budget"""
+        return self.total_budget() - self.total_hours_worked()
+
+    def is_before(self):
+        return self.start_date > datetime.date.today()
+
+    def is_complete(self):
+        return self.end_date < datetime.date.today()
+
+
 class AssignmentManager(models.Manager):
     def active_during_week(self, week, next_week):
         q = Q(contract__end_date__gte=week, contract__end_date__lt=next_week)
