@@ -1,5 +1,6 @@
 import urllib
 import datetime
+import time
 import calendar
 from decimal import Decimal
 
@@ -9,6 +10,7 @@ from django.db.models import Sum
 from django.core.urlresolvers import reverse
 
 from dateutil.relativedelta import relativedelta
+from dateutil import rrule
 
 from timepiece.models import PersonRepeatPeriod, AssignmentAllocation
 import timepiece.models as timepiece
@@ -205,38 +207,18 @@ def show_cal(from_date, offset=0):
     return html_cal.formatmonth(date.year, date.month)
 
 
-@register.inclusion_tag('timepiece/time-sheet/_invoice_row.html',
-                        takes_context=True)
-def build_invoice_row(context, entries, to_date, from_date):
-    hours_invoiced = hours_uninvoiced = 0
+@register.simple_tag
+def get_uninvoiced_hours(entries):
+    hours_uninvoiced = 0
     for entry in entries:
-        project = entry['project__pk']
-        if entry['status'] == 'invoiced':
-            hours_invoiced += entry['s']
-        else:
+        if entry['status'] != 'invoiced' and entry['status'] != 'not-invoiced':
             hours_uninvoiced += entry['s']
-    to_date_str = from_date_str = ''
-    if to_date:
-        to_date_str = to_date.strftime('%m/%d/%Y')
-    if from_date:
-        from_date_str = from_date.strftime('%m/%d/%Y')
-    csv_get_str = urllib.urlencode({
-        'to_date': to_date_str,
-        'from_date': from_date_str,
-        'status': 'approved',
-    })
-    csv_url = reverse('export_project_time_sheet', args=[project, ])
-    csv_url += '?' + csv_get_str
-    invoice_get_str = urllib.urlencode({
-        'to_date': to_date_str,
-        'from_date': from_date_str,
-        'project': project,
-    })
-    invoice_url = reverse('time_sheet_change_status', args=['invoice', ])
-    invoice_url += '?' + invoice_get_str
-    return {
-        'hours_invoiced': hours_invoiced,
-        'hours_uninvoiced': hours_uninvoiced,
-        'csv_url': csv_url,
-        'invoice_url': invoice_url,
-        }
+    return hours_uninvoiced
+
+
+@register.filter
+def work_days(end):
+    weekdays = (rrule.MO, rrule.TU, rrule.WE, rrule.TH, rrule.FR)
+    days = rrule.rrule(rrule.DAILY, byweekday=weekdays,
+                       dtstart=datetime.date.today(), until=end)
+    return len(list(days))
