@@ -294,14 +294,26 @@ class ClockInTest(TimepieceDataTestCase):
 
     def testClockInUnauthorizedProject(self):
         self.client.login(username='user', password='abc')
-        data = {
-            'project': self.project2.id,
-            'start_time_0': [u'11/02/2009'],
-            'start_time_1': [u'11:09:21'],
-        }
+        data = self.clock_in_form
+        data.update({'project': self.project2.id})
         response = self.client.post(self.url, data)
         self.assertEqual(response.status_code, 200)
         self.assertTrue(response.context['form'].errors)
+        err_msg = 'Select a valid choice. That choice is not one of the ' + \
+                  'available choices.'
+        self.assertFormError(response, 'form', 'project', err_msg)
+
+    def testClockInBadActivity(self):
+        self.client.login(username='user', password='abc')
+        data = self.clock_in_form
+        data.update({
+            'project': self.project.id,
+            'activity': self.sick_activity.id,
+        })
+        response = self.client.post(self.url, data)        
+        err_msg = 'sick/personal is not allowed for this project. Please '
+        err_msg += 'choose among development, and Work'
+        self.assertFormError(response, 'form', None, err_msg)
 
 
 class AutoActivityTest(TimepieceDataTestCase):
@@ -725,22 +737,6 @@ class CreateEditEntry(TimepieceDataTestCase):
             '%(project)s - %(activity)s starting at %(st_str)s' % \
             self.current_entry_data)
 
-    def testCreateBlockByFuture(self):
-        """
-        Test that add entry is blocked if the end time is in the future
-        """
-        five_min_later = self.now + datetime.timedelta(minutes=5)
-        future_entry = self.default_data
-        future_entry.update({
-            'start_time_0': self.now.strftime('%m/%d/%Y'),
-            'start_time_1': self.now.strftime('%H:%M:%S'),
-            'end_time_0': five_min_later.strftime('%m/%d/%Y'),
-            'end_time_1': five_min_later.strftime('%H:%M:%S'),
-        })
-        response = self.client.post(self.create_url, future_entry, follow=True)
-        self.assertFormError(response, 'form', None,
-            'Entries may not be added in the future.')
-
     def testProjectList(self):
         """
         Make sure the list of available projects conforms to user associations
@@ -751,6 +747,18 @@ class CreateEditEntry(TimepieceDataTestCase):
         self.assertTrue(self.project in projects)
         self.assertFalse(self.project2 in projects)
 
+    def testBadActivity(self):
+        """
+        Make sure the user cannot add an entry for an activity that is not in
+        the project's activity group
+        """
+        data = self.default_data
+        data.update({'activity': self.sick_activity.id})
+        response = self.client.post(self.create_url, data)
+        err_msg = 'sick/personal is not allowed for this project. Please '
+        err_msg += 'choose among development, and Work'
+        self.assertFormError(response, 'form', None, err_msg)
+
 
 class StatusTest(TimepieceDataTestCase):
     def setUp(self):
