@@ -50,7 +50,6 @@ urlpatterns = patterns('',
         views.create_edit_business,
         name='edit_business',
     ),
-
     url(r'^project/list/$', views.list_projects, name='list_projects'),
     url(
         r'^project/(?P<project_id>\d+)/$',
@@ -115,13 +114,12 @@ urlpatterns = patterns('',
         name='view_person_time_sheet',
     ),
     url(
-        r'^time-sheet/(?P<action>verify|approve|invoice)/' +
+        r'^time-sheet/(?P<action>verify|approve)/' +
         r'(?:(?P<person_id>\d+)/)?(?:(?P<period_id>\d+)/)?' +
         r'(?:(?P<window_id>\d+)/)?$',
         views.time_sheet_change_status,
         name='time_sheet_change_status',
     ),
-
     url(
         r'^time-sheet/project/(?P<project_id>\d+)/(?:(?P<window_id>\d+)/)?$',
         views.project_time_sheet,
@@ -133,12 +131,6 @@ urlpatterns = patterns('',
         views.export_project_time_sheet,
         name='export_project_time_sheet',
     ),
-    url(
-        r'^time-sheet/project/invoices/$',
-        views.invoice_projects,
-        name='invoice_projects',
-    ),
-
     url(
         r'^projection/$',
         views.projection_summary,
@@ -149,4 +141,53 @@ urlpatterns = patterns('',
         views.edit_settings,
         name='edit_settings',
     ),
+
+    ### Invoices ###
+    url(
+        r'^invoice/outstanding/$',
+        views.invoice_projects,
+        name='invoice_projects',
+    ),
+    url(
+        r'invoice/project/(?P<project_id>\d+)/create/' +
+        r'(?P<to_date>\d\d\d\d-\d\d-\d\d)/' +
+        r'(?:(?P<from_date>\d\d\d\d-\d\d-\d\d)/)?$',
+        views.confirm_invoice_project,
+        name='confirm_invoice_project',
+    ),
+    url(
+        r'^invoice/list/$',
+        views.InvoiceList.as_view(),
+        name='list_invoices',
+    ),
+    url(
+        r'^invoice/(?P<pk>\d+)/$',
+        views.InvoiceDetail.as_view(),
+        name='view_invoice'
+    ),
+    url(
+        r'^invoice/(?P<pk>\d+)/csv/$',
+        views.InvoiceCSV.as_view(),
+        name='view_invoice_csv'
+    ),
+    url(
+        r'^invoice/edit/(?P<pk>\d+)/$',
+        views.InvoiceEdit.as_view(),
+        name='edit_invoice',
+    ),
+    url(
+        r'^invoice/delete/(?P<pk>\d+)/$',
+        views.InvoiceDelete.as_view(),
+        name='delete_invoice',
+    ),
+    url(
+        r'^invoice/remove-entry/(?P<invoice_id>\d+)/(?P<entry_id>\d+)/$',
+        views.remove_invoice_entry,
+        name='remove_invoice_entry',
+    ),
+    url(
+        r'^contract/(?P<pk>\d+)/$',
+        views.ContractDetail.as_view(),
+        name='view_contract',
+    ),
 )
